%#########################################################################################
%#
%#                   function ELLIPS
%#
%#     displays tensors as ellipsoids. The color is chosen by the 
%#     average eigen value on the actual colormap.
%#
%#     usage ellips(n,d,v,dmax,xoffset,yoffset)
%#
%#            n  = point resolution of graphics 
%#            d  = (3x1) vector of the eigen values 
%#            v  = (3x3) matrix of the eigen vectors
%#         dmax  = maximum eigen value
%#      xoffset  = offset of x-axis of plot for continous display
%#      yoffset  = offset of y-axis of plot for continous display
%#            
%#       (c) P. Blmler 2/2003
%#########################################################################################
%----------------------------------------------------------------------------
%  version 1.1 PB 5/2/04    (abs in line 27)
%----------------------------------------------------------------------------



function result=ellips(n,d,v,dmax,xoffset,yoffset)
v=reshape(v,3,3);
d=abs(d)/dmax;
cmap=colormap;
colorvalue=round(length(cmap)*mean(d));
[dummy,index]=sort(d);
di=ones(3,1)/n*4;
for i=1:3
   di(index(i))=d(index(i));
   if di(index(i)) < 1/n
       di(index(i))=1/n;
   end
end

[X,Y,Z]=sphere(n-1);
Xd=X*di(1);Yd=Y*di(2);Zd=Z*di(3); %largest d value
for i=1:n
    z=[Xd(:,i),Yd(:,i),Zd(:,i)]*v;
    Xd(:,i)=z(:,1);
    Yd(:,i)=z(:,2);
    Zd(:,i)=z(:,3);
end
result=surf(Xd+xoffset,Yd+yoffset,Zd,'EdgeColor','none','FaceColor',cmap(colorvalue,:));
axis equal; axis off;
