%#############################################
%                                            #
%           function linfit(x,y)             #
%                                            #
% performs a linear fit to the data vector y #
% as a function of vector x
%                                            #
% dimensions of x,y have to be equal         #
%                                            #
% fit function: y=a*x+b                      #
%                                            #
% output is: vector [a,b,da,db,chi,r]            #
% where dm, db are the errors of slope and   #
% y-intercept and c is chi square by         #
% number(degrees of freedom)                 #
%                                            #
%#############################################

function ret_v=linfit(x,y)

sxy=sum((x-mean(x)).*(y-mean(y)));
sxx=sum((x-mean(x)).*(x-mean(x)));
syy=sum((y-mean(y)).*(y-mean(y)));
Nx=length(x);
Ny=length(y);
if Nx ~= Ny
    disp('ERROR: vectors have different size!');
    return
end

a=sxy/sxx;
b=mean(y)-a*mean(x);
s=sqrt((syy-a*sxy)/(Nx-2));
da=s/sqrt(sxx);
db=s*sqrt(1/Nx+mean(x)^2/sxx);
r=sqrt(sxy^2/sxx/syy);
ret_v=[a,b,da,db,r];