%#########################################################################################
%#
%#                   function UFO
%#
%#     displays tensors in a special 'UFO' way (largest eigen value = spike,
%#     middle eigen value = disk, smallest eigen value = sphere). The color 
%#     is chosen by the average eigen value on the actual colormap.
%#
%#     usage ufo(n,d,v,dmax,xoffset,yoffset)
%#
%#            n  = point resolution of graphics 
%#            d  = (3x1) vector of the eigen values 
%#            v  = (3x3) matrix of the eigen vectors
%#         dmax  = maximum eigen value
%#      xoffset  = offset of x-axis of plot for continous display
%#      yoffset  = offset of y-axis of plot for continous display
%#            
%#       (c) P. Blmler 2/2003
%#########################################################################################
%----------------------------------------------------------------------------
%  version 1.1 PB 5/2/04    (abs in line 27)
%----------------------------------------------------------------------------



function result=ufo(n,d,v,dmax,xoffset,yoffset)
v=reshape(v,3,3);
d=abs(d)/dmax;
cmap=colormap;
colorvalue=round(length(cmap)*mean(d));
[dummy,index]=sort(d);
di=ones(3)/n*4;
biggest=index(3);
di(biggest,1)=d(biggest);
switch index(2)
case 1,di(2,2)=d(index(2));
       di(3,2)=d(index(2));
case 2,di(1,2)=d(index(2));
       di(3,2)=d(index(2));
case 3,di(1,2)=d(index(2));
       di(2,2)=d(index(2));
end
di(:,3)=d(index(1));

[X,Y,Z]=sphere(n-1);
X1=X*di(1,1);Y1=Y*di(2,1);Z1=Z*di(3,1); %largest d value
X2=X*di(1,2);Y2=Y*di(2,2);Z2=Z*di(3,2); %middle d value
X3=X*di(1,3);Y3=Y*di(2,3);Z3=Z*di(3,3); %smallest d value
null=zeros(n,1);
Xd=[X1,null,X2,null,X3];
Yd=[Y1,null,Y2,null,Y3];
Zd=[Z1,null,Z2,null,Z3];
for i=1:3*n+2
    z=[Xd(:,i),Yd(:,i),Zd(:,i)]*v;
    Xd(:,i)=z(:,1);
    Yd(:,i)=z(:,2);
    Zd(:,i)=z(:,3);
end
result=surf(Xd+xoffset,Yd+yoffset,Zd,'EdgeColor','none','FaceColor',cmap(colorvalue,:));
axis equal;axis off